;*****************************************************************************************
;  Glowing Ember
;
;  Nintendo has some hints on how to conserve battery power so the little kiddies will
;  be able to play longer. This is good info for making the red LED power light flicker
;  like a hot coal! By enabling all audio/ IR port/ Graphics, the LED becomes dimmer.
;  By turning them all off the LED becomes brighter. Change the rate between turning them
;  on and off and you can create a flicker. This demo will take a random number for the delay
;  to create the flicker of a glowing hot ember. 
;
;  All Code by -Pan- of Anthrox  (C) 1999
;  Use as You wish, modify if You want
;  For more cool, unique gags and gadgets for GameBoy visit:
;  http://www.jps.net/paninaro
;  http://www.anthrox.com
;*****************************************************************************************

SECTION "Main Mem", BSS[$C000]

timerh: ds 1
timerl: ds 1
rndoffseth: ds 1
rndoffsetl: ds 1
flip: ds 1


         



    ;******************************************************************
    ;                            Header Equates
    ;******************************************************************
;Cartridge type:
; 0 - Rom only      3 - ROM+MBC1+RAM+Battery  1 - ROM+MBC1      5 - ROM+MBC2
; 2 - ROM+MBC1+RAM  6 - ROM+MBC2+Battery
CARTTYPE  EQU 2
;Rom Size:
; 0 - 256kBit =  32kB =  2 banks ; 1 - 512kBit =  64kB =  4 banks
; 2 -   1MBit = 128kB =  8 banks ; 3 -   2MBit = 256kB = 16 banks
; 4 -   4MBit = 512kB = 32 banks
ROMSIZE   EQU 0
;Ram Size:
; 0 - None ; 1 -  16kBit =  2kB = 1 bank
; 3 - 256kBit = 32kB = 4 banks
RAMSIZE   EQU 3
;Manufacturer code (word):
MANUCODE  EQU 1
;Version Number (byte):
VERSION   EQU 1

;************************************************************************************
;     Do NOT modify unless you are familiar with header info *Modified for Game Boy Color*
;************************************************************************************

SECTION "Header",home[$0100]
  ;Jumper
  nop
  jp Start ; go to Start of program:
  ;Nintendo Scrolling Title Graphic
  DB $CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
  DB $00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
  DB $BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E
  ;Title of the game
  db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128 ; Filled in by RGBFIX
  ;Not used
  db 0,0,0; last should be 3, won't work in emu unles it's 0
  ;Cartridge type:
  db CARTTYPE
  ;Rom Size:
  db ROMSIZE
  ;Ram Size:
  db RAMSIZE
  ;Manufacturer code:
  db 00, $33
  ;Version Number
  db VERSION
  ;Complement check
  db $0a
  ;Checksum
  dw 0





  ;*********************************************************************
  ;          This is the start of the program
  ;*********************************************************************

 ;SECTION "Main", home

Start:
  
   ld  SP,$FFF4          ; Init stack ptr



   ld a,0
   ld [timerh], a
   ld [timerl], a
   ld [rndoffseth], a
   ld [rndoffsetl], a
   ld [flip], a
   
  

;===============================================================================
;                  This makes the red LED light flicker
;===============================================================================
  
megaloop:

  ld hl,randomdata          ; get address of random data
  ld a,[rndoffseth]         ; fetch offset
  ld e, a                   ; store it in E
  ld d,0                    ; clear D 
  add hl, de                ; add rnddata address with offset in de, leave in HL
  ld a, [hl]                ; get rnd data
  ld [timerh], a            ; store it in Timer Hi  


  ld hl,randomdata          ; get address of random data
  ld a,[rndoffsetl]         ; fetch offset
  and $F                    ; get only least 4 bits ($0x)
  swap a                    ; make $0x into $x0
  ld e, a                   ; store it in E
  ld d,0                    ; clear D 
  add hl, de                ; add rnddata address with offset in de, leave in HL
  ld a, [hl]                ; get rnd data
  ld [timerl], a            ; store it in Timer Lo


  ld a, [timerl]            ; put Timer Lo in B
  ld b, a
  ld a, [timerh]            ; put Timer Hi in A

timerloop:
  dec b
  jp nz, timerloop          ; this is a timer delay, the timer is random data
  dec a                     
  jp nz, timerloop
    
  
  ld a, [flip]
  xor $ff                    ; enable stuff/disable stuff (audio, IR Port, video)
  ld [flip], a
  
  ld a, [flip]
  ldio [$24], a
  ldio [$25], a
  ldio [$26], a
  ldio [$21], a
  ldio [$22], a
  ldio [$20], a
  ldio [$1a], a
  ldio [$1b], a
  ldio [$1c], a
  ldio [$1d], a
  ldio [$1e], a
  ldio [$16], a
  ldio [$17], a
  ldio [$18], a
  ldio [$19], a
  ldio [$13], a
  ldio [$14], a
  ldio [$11], a
  ldio [$12], a
  ldio [$10], a
  ldio [$56], a
  ldio [$40], a

  ld a, $01
  ldio [$4d], a    ; switch between normal and double speed
  stop

  ld a, [rndoffseth]
  inc a                  ; increase rnd data offset (for Timer Hi)
  ld [rndoffseth], a
  
  
  ld a, [rndoffsetl]
  inc a                  ; increase rnd data offset (for Timer Lo)
  inc a
  inc a
  ld [rndoffsetl], a
   
   
  jp megaloop             ; Go back and do it all over again!
   
    
;=============================================================================
;                          The rest is all data
;=============================================================================
 

randomdata:
db  208,  10,  135,  8,  77,  31,  240,  19
db  152,  52,  9,  11,  52,  28,  99,  5
db  107,  65,  209,  88,  204,  71,  126,  93
db  223,  235,  56,  115,  19,  51,  11,  71
db  241,  192,  20,  82,  160,  134,  146,  163
db  130,  160,  65,  222,  58,  126,  227,  134
db  211,  66,  34,  119,  67,  104,  146,  182
db  15,  171,  26,  110,  71,  40,  116,  51
db  188,  188,  18,  136,  114,  179,  35,  168
db  79,  164,  244,  129,  24,  152,  126,  252
db  153,  106,  55,  134,  43,  115,  42,  200
db  83,  102,  74,  185,  198,  120,  10,  215
db  152,  196,  40,  177,  169,  238,  201,  169
db  103,  9,  204,  248,  180,  177,  90,  242
db  37,  132,  185,  140,  108,  176,  106,  28
db  24,  226,  114,  95,  124,  109,  242,  191
db  235,  162,  1,  214,  48,  128,  175,  48
db  50,  145,  112,  78,  249,  20,  156,  238
db  220,  87,  82,  146,  239,  103,  126,  55
db  194,  235,  59,  102,  77,  78,  90,  114
db  34,  29,  67,  254,  235,  174,  250,  193
db  25,  11,  134,  136,  207,  6,  113,  122
db  42,  170,  167,  159,  158,  223,  140,  162
db  189,  71,  74,  213,  34,  98,  102,  119
db  160,  250,  150,  51,  197,  195,  213,  231
db  129,  58,  184,  175,  30,  206,  252,  27
db  115,  71,  99,  186,  94,  98,  189,  229
db  111,  188,  73,  182,  225,  241,  62,  86
db  207,  8,  163,  121,  165,  7,  103,  44
db  212,  231,  174,  204,  202,  183,  106,  90
db  35,  243,  44,  238,  25,  55,  57,  138
db  64,  22,  223,  18,  119,  71,  11,  151

 